/****************************************************************************
** Meta object code from reading C++ file 'dpalette.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../include/kernel/dpalette.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dpalette.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3Dtk3Gui8DPaletteE_t {};
} // unnamed namespace

template <> constexpr inline auto Dtk::Gui::DPalette::qt_create_metaobjectdata<qt_meta_tag_ZN3Dtk3Gui8DPaletteE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Dtk::Gui::DPalette",
        "ColorType",
        "NoType",
        "ItemBackground",
        "TextTitle",
        "TextTips",
        "TextWarning",
        "TextLively",
        "LightLively",
        "DarkLively",
        "FrameBorder",
        "PlaceholderText",
        "FrameShadowBorder",
        "ObviousBackground",
        "NColorTypes"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'ColorType'
        QtMocHelpers::EnumData<enum ColorType>(1, 1, QMC::EnumFlags{}).add({
            {    2, ColorType::NoType },
            {    3, ColorType::ItemBackground },
            {    4, ColorType::TextTitle },
            {    5, ColorType::TextTips },
            {    6, ColorType::TextWarning },
            {    7, ColorType::TextLively },
            {    8, ColorType::LightLively },
            {    9, ColorType::DarkLively },
            {   10, ColorType::FrameBorder },
            {   11, ColorType::PlaceholderText },
            {   12, ColorType::FrameShadowBorder },
            {   13, ColorType::ObviousBackground },
            {   14, ColorType::NColorTypes },
        }),
    };
    return QtMocHelpers::metaObjectData<DPalette, qt_meta_tag_ZN3Dtk3Gui8DPaletteE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Dtk::Gui::DPalette::staticMetaObject = { {
    QtPrivate::MetaObjectForType<QPalette>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Dtk3Gui8DPaletteE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Dtk3Gui8DPaletteE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3Dtk3Gui8DPaletteE_t>.metaTypes,
    nullptr
} };

QT_WARNING_POP
