/****************************************************************************
** Meta object code from reading C++ file 'personalizationwaylandclientextension.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../src/plugins/platform/treeland/personalizationwaylandclientextension.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'personalizationwaylandclientextension.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3Dtk3Gui22PersonalizationManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto Dtk::Gui::PersonalizationManager::qt_create_metaobjectdata<qt_meta_tag_ZN3Dtk3Gui22PersonalizationManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Dtk::Gui::PersonalizationManager"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PersonalizationManager, qt_meta_tag_ZN3Dtk3Gui22PersonalizationManagerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Dtk::Gui::PersonalizationManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QWaylandClientExtensionTemplate<PersonalizationManager>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Dtk3Gui22PersonalizationManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Dtk3Gui22PersonalizationManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3Dtk3Gui22PersonalizationManagerE_t>.metaTypes,
    nullptr
} };

void Dtk::Gui::PersonalizationManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PersonalizationManager *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *Dtk::Gui::PersonalizationManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Dtk::Gui::PersonalizationManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Dtk3Gui22PersonalizationManagerE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QtWayland::treeland_personalization_manager_v1"))
        return static_cast< QtWayland::treeland_personalization_manager_v1*>(this);
    return QWaylandClientExtensionTemplate<PersonalizationManager>::qt_metacast(_clname);
}

int Dtk::Gui::PersonalizationManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWaylandClientExtensionTemplate<PersonalizationManager>::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN3Dtk3Gui28PersonalizationWindowContextE_t {};
} // unnamed namespace

template <> constexpr inline auto Dtk::Gui::PersonalizationWindowContext::qt_create_metaobjectdata<qt_meta_tag_ZN3Dtk3Gui28PersonalizationWindowContextE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Dtk::Gui::PersonalizationWindowContext"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PersonalizationWindowContext, qt_meta_tag_ZN3Dtk3Gui28PersonalizationWindowContextE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Dtk::Gui::PersonalizationWindowContext::staticMetaObject = { {
    QMetaObject::SuperData::link<QWaylandClientExtensionTemplate<PersonalizationWindowContext>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Dtk3Gui28PersonalizationWindowContextE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Dtk3Gui28PersonalizationWindowContextE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3Dtk3Gui28PersonalizationWindowContextE_t>.metaTypes,
    nullptr
} };

void Dtk::Gui::PersonalizationWindowContext::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PersonalizationWindowContext *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *Dtk::Gui::PersonalizationWindowContext::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Dtk::Gui::PersonalizationWindowContext::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Dtk3Gui28PersonalizationWindowContextE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QtWayland::treeland_personalization_window_context_v1"))
        return static_cast< QtWayland::treeland_personalization_window_context_v1*>(this);
    return QWaylandClientExtensionTemplate<PersonalizationWindowContext>::qt_metacast(_clname);
}

int Dtk::Gui::PersonalizationWindowContext::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWaylandClientExtensionTemplate<PersonalizationWindowContext>::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN3Dtk3Gui32PersonalizationAppearanceContextE_t {};
} // unnamed namespace

template <> constexpr inline auto Dtk::Gui::PersonalizationAppearanceContext::qt_create_metaobjectdata<qt_meta_tag_ZN3Dtk3Gui32PersonalizationAppearanceContextE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Dtk::Gui::PersonalizationAppearanceContext"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PersonalizationAppearanceContext, qt_meta_tag_ZN3Dtk3Gui32PersonalizationAppearanceContextE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Dtk::Gui::PersonalizationAppearanceContext::staticMetaObject = { {
    QMetaObject::SuperData::link<QWaylandClientExtensionTemplate<PersonalizationAppearanceContext>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Dtk3Gui32PersonalizationAppearanceContextE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Dtk3Gui32PersonalizationAppearanceContextE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3Dtk3Gui32PersonalizationAppearanceContextE_t>.metaTypes,
    nullptr
} };

void Dtk::Gui::PersonalizationAppearanceContext::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PersonalizationAppearanceContext *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *Dtk::Gui::PersonalizationAppearanceContext::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Dtk::Gui::PersonalizationAppearanceContext::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Dtk3Gui32PersonalizationAppearanceContextE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QtWayland::treeland_personalization_appearance_context_v1"))
        return static_cast< QtWayland::treeland_personalization_appearance_context_v1*>(this);
    return QWaylandClientExtensionTemplate<PersonalizationAppearanceContext>::qt_metacast(_clname);
}

int Dtk::Gui::PersonalizationAppearanceContext::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWaylandClientExtensionTemplate<PersonalizationAppearanceContext>::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN3Dtk3Gui26PersonalizationFontContextE_t {};
} // unnamed namespace

template <> constexpr inline auto Dtk::Gui::PersonalizationFontContext::qt_create_metaobjectdata<qt_meta_tag_ZN3Dtk3Gui26PersonalizationFontContextE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Dtk::Gui::PersonalizationFontContext"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PersonalizationFontContext, qt_meta_tag_ZN3Dtk3Gui26PersonalizationFontContextE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Dtk::Gui::PersonalizationFontContext::staticMetaObject = { {
    QMetaObject::SuperData::link<QWaylandClientExtensionTemplate<PersonalizationFontContext>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Dtk3Gui26PersonalizationFontContextE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Dtk3Gui26PersonalizationFontContextE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3Dtk3Gui26PersonalizationFontContextE_t>.metaTypes,
    nullptr
} };

void Dtk::Gui::PersonalizationFontContext::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PersonalizationFontContext *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *Dtk::Gui::PersonalizationFontContext::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Dtk::Gui::PersonalizationFontContext::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Dtk3Gui26PersonalizationFontContextE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QtWayland::treeland_personalization_font_context_v1"))
        return static_cast< QtWayland::treeland_personalization_font_context_v1*>(this);
    return QWaylandClientExtensionTemplate<PersonalizationFontContext>::qt_metacast(_clname);
}

int Dtk::Gui::PersonalizationFontContext::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWaylandClientExtensionTemplate<PersonalizationFontContext>::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
